@@ -23,6 +23,7 @@
 
 #include <QMouseEvent>
 #include <QApplication>
+#include <QMimeData>
 
 namespace Konsole {
 
@@ -31,18 +32,10 @@ DetachableTabBar::DetachableTabBar(QWidget *parent) :
     dragType(DragType::NONE),
     _originalCursor(cursor()),
     tabId(-1)
-{}
-
-bool DetachableTabBar::droppedContainerIsNotThis(const QPoint& currentPos) const
 {
-    for(const auto dropWidget : _containers) {
-        if (dropWidget->rect().contains(dropWidget->mapFromGlobal(currentPos))) {
-            if (dropWidget != parent()) {
-                return true;
-            }
-        }
-    }
-    return false;
+    setAcceptDrops(true);
+    setUsesScrollButtons(false);
+    setElideMode(Qt::TextElideMode::ElideMiddle);
 }
 
 void DetachableTabBar::middleMouseButtonClickAt(const QPoint& pos)
@@ -66,16 +59,9 @@ void DetachableTabBar::mouseMoveEvent(QMouseEvent *event)
     auto widgetAtPos = qApp->topLevelAt(event->globalPos());
     if (widgetAtPos != nullptr) {
         if (window() == widgetAtPos->window()) {
-            if (droppedContainerIsNotThis(event->globalPos())) {
-                if (dragType != DragType::WINDOW) {
-                    dragType = DragType::WINDOW;
-                    setCursor(QCursor(Qt::DragMoveCursor));
-                }
-            } else {
-                if (dragType != DragType::NONE) {
-                    dragType = DragType::NONE;
-                    setCursor(_originalCursor);
-                }
+            if (dragType != DragType::NONE) {
+                dragType = DragType::NONE;
+                setCursor(_originalCursor);
             }
         } else {
             if (dragType != DragType::WINDOW) {
@@ -125,14 +111,29 @@ void DetachableTabBar::mouseReleaseEvent(QMouseEvent *event)
             emit detachTab(currentIndex());
         }
     } else if (window() != widgetAtPos->window()) {
-        // splits have a tendency to break, forbid to detach if split and it's the last tab.
         if (_containers.size() == 1 || count() > 1) {
             emit moveTabToWindow(currentIndex(), widgetAtPos);
         }
-    } else if (droppedContainerIsNotThis(event->globalPos())){
-        if (count() != 1) {
-            emit moveTabToWindow(currentIndex(), widgetAtPos);
-        }
+    }
+}
+
+void DetachableTabBar::dragEnterEvent(QDragEnterEvent* event)
+{
+    const auto dragId = QStringLiteral("konsole/terminal_display");
+    if (event->mimeData()->hasFormat(dragId)) {
+        auto other_pid = event->mimeData()->data(dragId).toInt();
+        // don't accept the drop if it's another instance of konsole
+        if (qApp->applicationPid() != other_pid)
+            return;
+        event->accept();
+    }
+}
+
+void DetachableTabBar::dragMoveEvent(QDragMoveEvent* event)
+{
+    int tabIdx = tabAt(event->pos());
+    if (tabIdx != -1) {
+        setCurrentIndex(tabIdx);
     }
 }
 
